# -*- coding: utf-8 -*-

import logging
from .utils import Counter
from .base import Base
from .exceptions import ParameterError
from .handler import Subscriber, message_center

logger = logging.getLogger("sdk.gomer.audio")


class Audio(Base):
    """
    audio, contains the interface related to sound, TTS, music production, sound, play, stop and other functions
    """

    __slots__ = ("_text", "_file_name", "_speed", "_misic", "_src_file", "_convert_type", "_dst_file", "_vol")

    def __init__(self):
        self._speed = 4
        self._vol = 2
        self._file_name = None
        self._convert_type = None

    @property
    def convert_type(self):
        return self._convert_type

    @convert_type.setter
    def convert_type(self, value):
        if value is None:
            if self.convert_type is None:
                logger.error("Parameter convert_type is None error")
                raise ParameterError("Parameter convert_type is None error")
            value = self.convert_type
        self._convert_type = value

    @property
    def vol(self):
        return self._vol

    @vol.setter
    def vol(self, value):
        if not value:
            value = self.vol
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter vol type error")
            raise ParameterError("Parameter vol type error")
        if value < 0 or value > 5:
            logger.error("Parameter vol out of range (0~5) error")
            raise ParameterError("Parameter vol out of range (0~5) error")
        self._vol = value

    @property
    def file_name(self):
        return self._file_name

    @file_name.setter
    def file_name(self, value):
        if value is None:
            if self.file_name is None:
                logger.error("Parameter file_name is None error")
                raise ParameterError("Parameter file_name is None error")
            value = self.file_name
        self._file_name = value

    @property
    def speed(self):
        return self._speed

    @speed.setter
    def speed(self, value):
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter speed type error")
            raise ParameterError("Parameter speed type error")
        if value < 1 or value > 8:
            logger.error("Parameter speed out of range (1~8) error")
            raise ParameterError("Parameter speed out of range (1~8) error")
        self._speed = value

    def make_tts(self, text):
        item = 1201
        if len(text) > 128:
            logger.error("Parameter text is too long error")
            raise ParameterError("Parameter text is too long error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=text, prmstr2=self.file_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="make_tts", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def make_music(self, music):
        item = 1202
        if len(music) > 128:
            logger.error("Parameter music is too long error")
            raise ParameterError("Parameter music is too long error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item,prm2=self.speed, prmstr1=music, prmstr2=self.file_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="make_music", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def convert_audio(self, src_file, dst_file):
        item = 1203
        if not src_file.endswith('.wav'):
            logger.error("src_file format not .wav error")
            raise ParameterError("src_file format not .wav error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_transmit_msg(seq, 101, src_file, self._get_file_size(src_file))
        subscriber = Subscriber(file_path=src_file, seq=seq, num=num, item=item, name="convert_audio",
                                msg_center=message_center)
        subscriber.subscribe()
        subscriber.send_msg(msg)
        if subscriber.get_result():
            subscriber.can_upload = 0
            subscriber.file_upload()
            if subscriber.get_result():
                subscriber.upload = 0
                seq = Counter().get_seq()
                msg = self._edit_msg(seq=seq, num=num, item=item, prm2 = self.convert_type,
                                     prmstr1=self._get_file_name(src_file), prmstr2 = dst_file)
                subscriber.seq = seq
                subscriber.send_msg(msg)
                return subscriber.get_result()
            else:
                logger.warning("File upload failed")
                return -1
        else:
            logger.warning("File cannot be uploaded")
            return -1

    def get_volume(self):
        item = 1204
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=102)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="get_volume", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def set_volume(self):
        item = 1204
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm2=self.vol)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="set_volume", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def get_player_status(self):
        item = 1205
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="get_player_status", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def play_audio(self):
        item = 1206
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=self.file_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="play_audio", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def stop_player(self):
        item = 1207
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=self.file_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="stop_player", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result


