# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import message_center, Subscriber

logger = logging.getLogger("sdk.gomer.face")


class Face(Base):
    """This paper introduces the interface of face recognition, which can open the data of many face algorithms to users,
       and can use the data to perform re-operation and create new ideas."""

    __slots__ = ("_image_type", "_min_face_size", "_max_face_size", "_min_face_score", "_display", "_face_index")

    display_dict = {
        True: 101,
        False: 100
    }

    image_type_dict = {
        "camera": 100,
        "image_array": 101,
        "file": 102
    }

    def __init__(self):
        self._image_type = "camera"
        self._min_face_size = 10
        self._max_face_size = 300
        self._min_face_score = 10
        self._display = True
        self._face_index = None

    @property
    def display(self):
        return self._display

    @display.setter
    def display(self, value):
        if value is None:
            value = self.display
        elif value not in (True, False):
            logger.error("Parameter display type error")
            raise ParameterError("Parameter display type error")
        self._display = value

    @property
    def face_index(self):
        return self._face_index

    @face_index.setter
    def face_index(self, value):
        if value is None:
            value = self.face_index
        self._face_index = value

    @property
    def min_face_score(self):
        return self._min_face_score

    @min_face_score.setter
    def min_face_score(self, value):
        if value is None:
            value = self.min_face_score
        else:
            try:
                value = int(value)
            except Exception:
                logger.error("Parameter min_face_score type error")
                raise ParameterError("Parameter min_face_score type error")
            if value < 0 or value > 100:
                logger.error("Parameter min_face_score out of range (0, 100) error")
                raise ParameterError("Parameter min_face_score out of range (0, 100) error")
        self._min_face_score = value

    @property
    def max_face_size(self):
        return self._max_face_size

    @max_face_size.setter
    def max_face_size(self, value):
        if value is None:
            value = self.max_face_size
        else:
            try:
                value = int(value)
            except Exception:
                logger.error("Parameter max_face_size type error")
                raise ParameterError("Parameter max_face_size type error")
            if value < 10 or value > 300:
                logger.error("Parameter max_face_size out of range (10, 300) error")
                raise ParameterError("Parameter max_face_size out of range (10, 300) error")
        self._max_face_size = value

    @property
    def min_face_size(self):
        return self._min_face_size

    @min_face_size.setter
    def min_face_size(self, value):
        if value is None:
            value = self.min_face_size
        else:
            try:
                value = int(value)
            except Exception:
                logger.error("Parameter min_face_size type error")
                raise ParameterError("Parameter min_face_size type error")
            if value < 10 or value > 300:
                logger.error("Parameter min_face_size out of range (10, 300) error")
                raise ParameterError("Parameter min_face_size out of range (10, 300) error")
        self._min_face_size = value

    @property
    def image_type(self):
        return self._image_type

    @image_type.setter
    def image_type(self, value):
        if value is None:
            value = self.image_type
        elif value not in ("camera", "image_array", "file"):
            logger.error("Parameter image_type type error")
            raise ParameterError("Parameter image_type type error")
        self._image_type = value

    def detect_face(self):
        item = 3101
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.image_type_dict.get(self.image_type),
                                                prm2=self.min_face_size, prm3=self.max_face_size,
                                                prm4=self.min_face_score, prm5=self.display_dict.get(self.display))
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="detect_face", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def get_face_feature(self):
        item = 3102
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm5=self.display_dict.get(self.display), prmstr1=self.face_index)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="get_face_feature", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def face_register(self, face_name):
        item = 3103
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=self.face_index, prmstr2=face_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="face_register",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def identity_recognize(self):
        item = 3104
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm5=self.display_dict.get(self.display), prmstr2=self.face_index)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="identity_recognize",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def expression_recognize(self):
        item = 3105
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm5=self.display_dict.get(self.display), prmstr2=self.face_index)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="expression_recognize",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def get_face_identity(self, identity_name):
        item = 3106
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm5=self.display_dict.get(self.display),
                                                prmstr1=self.face_index, prmstr2=identity_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="get_face_identity",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def get_face_similarity(self, identity_name1, identity_name2):
        item = 3107
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=identity_name1, prmstr2=identity_name2)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prm1", name="get_face_similarity",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def face_rename(self, old_name, new_name):
        item = 3108
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=old_name, prmstr2=new_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="face_rename",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def get_face_list(self):
        item = 3109
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, parameter="prmstr", name="get_face_list",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result


