# -*- coding: utf-8 -*-

import json
import os
import logging
import time
from ctypes import *
from threading import Thread, Timer
from . import config
from .utils import Counter, kill_process
from .utils import get_dict_value
from .exceptions import NoDevicesFound


logger = logging.getLogger("sdk.gomer.gproto")

recv_ack_seq_num = 0

base_path = os.path.dirname(os.path.split(os.path.realpath(__file__))[0])

PROTO_PATH = os.path.join(base_path, "dll_lib", "glproto64")

lib = CDLL(PROTO_PATH)


def search_socket_create():
    search_socket_id = lib.ProtUdpCreate()
    if search_socket_id == -1:
        logger.error("Failed to create search_socket")
        kill_process()
    logger.info("search_socket: %s was created successfully" % search_socket_id)
    return search_socket_id


def search_device(search_socket_id):
    """搜索设备"""
    lib.ProtSearchDevice(search_socket_id)


def select_device(did, sid):
    lib.ProtSelectDevice.argtypes = (c_char_p, c_int)
    lib.ProtSelectDevice.restype = c_int
    if lib.ProtSelectDevice(did.encode(), sid) != -1:
        logger.info("The connected device is %s" % did)
    else:
        logger.error("Failed to select device")
        raise ()


class SearchMsgRecvThread(Thread):
    def __init__(self, search_socket_id, device_list):
        Thread.__init__(self)
        self.search_socket_id = search_socket_id
        self.device_list = device_list

    def run(self):
        lib.ProtUdpRecv.argtypes = (c_int, c_char_p, c_int)
        lib.ProtUdpRecv.restype = c_int
        buff_r = create_string_buffer(1500, "\0")
        while True:
            lib.ProtUdpRecv(self.search_socket_id, buff_r, 1500)
            message = json.loads(buff_r.value.decode())
            logger.info("search_socket received msg:%s" % message)
            if get_dict_value(message, "call") is 101:
                self.device_list.append(get_dict_value(message, "did"))
                # config.DEVICE_LIST.append(get_dict_value(search_socket_recv_msg, "did"))
            elif get_dict_value(message, "call") is 103:
                config.HANDSHAKE_FLAG = True
            else:
                continue
            if config.SEARCH_SOCKET_DESTROY_FLAG:
                break


def send_search_msg(search_socket_id, msg):
    lib.ProtUdpSend.argtypes = (c_int, c_char_p, c_int)
    lib.ProtUdpSend.restype = c_int
    message = json.dumps(msg).encode()
    res = lib.ProtUdpSend(search_socket_id, message, len(message))
    print(res)
    if res != 0:
        logger.error("Error sending search_socket message!")
        kill_process()
    logger.info("search_socket sent message: %s" % message)


def search_socket_destroy(search_socket_id):
    res = lib.ProtUdpDestroy(search_socket_id)
    if res != 0:
        logger.error("Failed to delete search_socket")
    config.SEARCH_SOCKET_DESTROY_FLAG = True
    logger.info("Has been destroyed search_socket")


def communication_socket_create():
    communication_socket_id = lib.ProtMsgSocketCreate()
    print(communication_socket_id)
    if communication_socket_id == -1:
        logger.error("Failed to create communicate_socket")
        kill_process()
    logger.info("Created communicate_socket:%s" % communication_socket_id)
    return communication_socket_id


def communication_socket_destroy(communication_socket_id):
    res = lib.ProtMsgSocketDestroy(communication_socket_id)
    if res != 0:
        logger.error("Failed to destroy communication_socket")
        return
    logger.info("Has been destroyed communication_socket:%s" % communication_socket_id)


class CommunicationMsgRecvThread(Thread):
    def __init__(self, communication_socket_id):
        Thread.__init__(self)
        self.communication_socket_id = communication_socket_id

    def run(self):
        global recv_ack_seq_num
        lib.ProtMsgRecv.argtypes = (c_int, c_char_p, c_int)
        lib.ProtMsgRecv.restype = c_int
        buff_t = create_string_buffer(1500, "\0")
        while True:
            res = lib.ProtMsgRecv(self.communication_socket_id, buff_t, 1500)
            if -1 == res:
                logger.error("Error receiving communicate_socket message")
            recv_msg = json.loads(buff_t.value.decode())
            logger.info("communicate_socket received msg:%s" % recv_msg)
            seq = recv_msg.get("seq")
            if 2 == recv_msg.get("msgtype"):
                if not recv_msg.get("hard"):
                    if seq <= recv_ack_seq_num:
                        continue
                    recv_ack_seq_num = seq
            if 201 == get_dict_value(recv_msg, "call"):
                config.heart_queue.put(recv_msg)
            elif get_dict_value(recv_msg, "item"):
                if get_dict_value(recv_msg, "item") in config.EVENT_ITEMS:
                    config.event_queue.put(recv_msg)
                    msg = {"seq": get_dict_value(recv_msg, "seq"), "msgtype": 2, "code": 100}
                    send_communication_msg(msg, self.communication_socket_id)
                else:
                    logger.info("sdk_queue received msg:", recv_msg)
                    config.sdk_queue.put(recv_msg)
            elif get_dict_value(recv_msg, "mode") is 106:
                config.control_queue.put(recv_msg)
            elif get_dict_value(recv_msg, "type") is 202:
                # config.transmit_queue.put(recv_msg)
                config.sdk_queue.put(recv_msg)
                print("transmit_queue:", config.transmit_queue.qsize())
            else:
                continue


def send_communication_msg(msg, communication_socket_id):
    lib.ProtMsgSend.argtypes = (c_int, c_char_p, c_int)
    lib.ProtMsgSend.restype = c_int
    message = json.dumps(msg).encode()
    res = lib.ProtMsgSend(communication_socket_id, message, len(message))
    if res != 0:
        logger.error("Failed to send communicate_socket message1")
    logger.info("communicate_socket sent msg:%s" % message)


def file_upload(path):
    lib.ProtFileSend.argtypes = (c_char_p,)
    lib.ProtFileSend.restype = c_int
    res = lib.ProtFileSend(path.encode())
    if res != 0:
        logger.error("Failed to upload file")
    logger.info("Uploaded file:%s" % path)


class HeartBeatRecvThread(Thread):
    def __init__(self, communicate_socket_id):
        Thread.__init__(self)
        self.recv_heart_flag = 0
        self.heart_broken_flag = 0
        self.communicate_socket_id = communicate_socket_id
        self.timer2 = Timer(15, self.__heart_broken_timer)

    def __heart_broken_timer(self):
        logger.warning("Execute the heartbeat stop timer function")
        self.heart_broken_flag = 1

    def __speed_up_sending_heartbeat(self):
        logger.warning("Execute the heartbeat acceleration send timer function")
        for i in range(3):
            seq = Counter().get_seq()
            msg = {"seq": seq, "msgtype": 1, "hard": {"call": 200, "sid": config.SID}}
            send_communication_msg(msg, self.communicate_socket_id)
        self.timer2 = Timer(15, self.__heart_broken_timer)
        self.timer2.start()
        logger.info("Start timer2")

    def run(self):
        while True:
            if config.HEART_SEND_FLAG:
                logger.debug("Wait for a heartbeat response")
                config.HEART_SEND_FLAG = False
                timer1 = Timer(2, self.__speed_up_sending_heartbeat)
                timer1.start()
                logger.debug("Start timer1")
                while True:
                    if not config.heart_queue.empty():
                        if get_dict_value(config.heart_queue.get(), "call") is 201:
                            timer1.cancel()
                            self.timer2.cancel()
                            logger.debug("Destroy timer1,timer2")
                            self.recv_heart_flag = 1
                    if 1 == self.recv_heart_flag:
                        self.recv_heart_flag = 0
                        break
                    if 1 == self.heart_broken_flag:
                        logger.error("The heartbeat has been disconnected.")
                        kill_process()


class HeartBeatSendThread(Thread):
    def __init__(self, communicate_socket_id, sid):
        Thread.__init__(self)
        self.communicate_socket_id = communicate_socket_id
        self.sid = sid

    def run(self):
        while True:
            seq = Counter().get_seq()
            msg = {"seq": seq, "msgtype": 1, "hard": {"call": 200, "sid": self.sid}}
            send_communication_msg(msg, self.communicate_socket_id)
            config.HEART_SEND_FLAG = True
            logger.debug("PythonSDK has been sent heartbeat")
            time.sleep(10)

















