# -*- coding:utf-8 -*-

import logging
import threading
import os
import json
import time
from . import config
from .gproto import send_communication_msg, file_upload


logger = logging.getLogger("sdk.gomer.handler")


class Provider(object):
    def __init__(self):
        self.subscribers = []

    def subscribe(self, subscriber):
        self.subscribers.append(subscriber)

    def unsubscribe(self, subscriber):
        self.subscribers.remove(subscriber)

    def start_message_process(self, queue):
        t = threading.Thread(target=self._update, args=(queue,))
        t.start()

    def _update(self, queue):
        while True:
            msg = queue.get()
            for subscriber in self.subscribers:
                subscriber.run(msg)


class Subscriber(object):
    def __init__(self,seq, num, item, msg_center, name, file_path=None, parameter=None):
        self.seq = seq
        self.num = num
        self.item = item
        self.value_list = []
        self.parameter = parameter
        self.name = name
        self.ack = False
        self.result = None
        self.can_upload = 0
        self.provider = msg_center
        self.upload = 0
        self.file_path = file_path

    def subscribe(self):
        self.provider.subscribe(self)

    def unsubscribe(self):
        self.provider.unsubscribe(self)

    def get_result(self):
        start_time = time.time()
        while True:
            if self.result:
                self.unsubscribe()
                return self._result_parse()
            elif self.can_upload:
                if 1 == self.can_upload:
                    return True
                else:
                    return False
            elif self.upload:
                if 1 == self.upload:
                    return True
                else:
                    return False
            elif time.time() - start_time > 30:
                logger.warning("%s exec timeout" % self.name)
                return

    @staticmethod
    def send_msg(msg):
        send_communication_msg(msg, config.SOCKET_ID)

    def file_upload(self):
        file_upload(self.file_path)

    def run(self, msg):
        if self.seq == self._get_dict_value(msg, "seq") and 100 == self._get_dict_value(msg, "code"):
            if self._get_dict_value(msg, "status"):
                if 200 == self._get_dict_value(msg, "status"):  # Files can be uploaded
                    self.can_upload = 1
                    return
                else:
                    self.can_upload = -1   # Files can not be uploaded
                    return
            else:
                self.ack = True
                return
        elif self.seq != self._get_dict_value(msg, "seq") and 0 == self._get_dict_value(msg, "code"):
            self._send_ack_msg(msg)
            if self._get_dict_value(msg, "status") and self._get_dict_value(msg, "name") == self.file_path:
                if 202 == self._get_dict_value(msg, "status"):  # File uploaded successfully
                    self.upload = 1
                    return
                else:
                    self.upload = -1  # File upload failed
                    return
            elif self.item == self._get_dict_value(msg, "item") and 1 == self._get_dict_value(msg, "msgtype"):
                if self.num == self._get_dict_value(msg, "num") and self._get_dict_value(msg, "result") in (100, 101):
                    self.result = msg
                    return

    @staticmethod
    def _send_ack_msg(msg):
        msg = {"seq": msg.get("seq"), "msgtype": 2, "code": 100}
        send_communication_msg(msg, config.SOCKET_ID)

    def _get_dict_value(self, dict1, obj, default=None):
        for key, value in dict1.items():
            if key == obj:
                return value
            else:
                if type(value) is dict:
                    result = self._get_dict_value(value, obj)
                    if result is not default:
                        return result
        return default

    def _get_prmstr_value(self):
        for i in range(5):
            value_json = self._get_dict_value(self.result, "prmstr%s" % (i + 1))
            if value_json:
                value_dict = json.loads(value_json)
                key = list(value_dict.keys())[0]
                value_dict.get(key).insert(0, key)
                self.value_list.append(value_dict.get(key))
        return self.value_list

    def _result_parse(self):
        if 100 == self._get_dict_value(self.result, "result"):
            logger.info("%s execute successfully" % self.name)
            if not self.parameter:
                return 0
            elif self.parameter is "prm0":
                for i in ["prm%s" % i for i in range(1, 6)]:
                    self.value_list.append(self._get_dict_value(self.result, i))
                return self.value_list
            elif self.parameter in ["prm%s" % i for i in range(1, 6)]:
                self.value_list.append(self._get_dict_value(self.result, self.parameter))
                if 1 == len(self.value_list):
                    self.value_list = self.value_list[0]
                return self.value_list
            elif self.parameter is 'prmstr':
                return self._get_prmstr_value()
        else:
            logger.info("%s exec failure" % self.name)
            return -1

    def _get_file_name(self):
        return os.path.basename(self.file_path)


message_center = Provider()

