# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import message_center, Subscriber

logger = logging.getLogger("sdk.gomer.misc")


class Light(Base):
    """light class"""

    __slots__ = ("_rgb", "_mode", "_duration_time", "_light_time", "_dark_time")

    mode_dict = {
        "always-on": 0,
        "blink": 1,
        "breath": 2
    }

    def __init__(self):
        self._rgb = (50, 50, 50)
        self._mode = "blink"
        self._duration_time = 65535
        self._light_time = 1000
        self._dark_time = 1000
        # self.set_color()  # 初始化设置颜色

    @property
    def dark_time(self):
        return self._dark_time

    @dark_time.setter
    def dark_time(self, value):
        if value is None:
            if self.dark_time is None:
                logger.error("Parameter dark_time is None error")
                raise ParameterError("Parameter dark_time is None error")
            value = self.dark_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter dark_time type error")
            raise ParameterError("Parameter dark_time type error")
        if value < 0 or value > 10000:
            logger.error("Parameter dark_time out of range (0, 10000) error")
            raise ParameterError("Parameter dark_time out of range (0, 10000) error")
        self._dark_time = value

    @property
    def light_time(self):
        return self._light_time

    @light_time.setter
    def light_time(self, value):
        if value is None:
            if self.dark_time is None:
                logger.error("Parameter light_time is None error")
                raise ParameterError("Parameter light_time is None error")
            value = self.dark_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter light_time type error")
            raise ParameterError("Parameter light_time type error")
        if value < 0 or value > 10000:
            logger.error("Parameter light_time out of range (0, 10000) error")
            raise ParameterError("Parameter light_time out of range (0, 10000) error")
        self._light_time = value

    @property
    def duration_time(self):
        return self._duration_time

    @duration_time.setter
    def duration_time(self, value):
        if value is None:
            if self.duration_time is None:
                logger.error("Parameter duration_time is None error")
                raise ParameterError("Parameter duration_time is None error")
            value = self.duration_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter duration_time type error")
            raise ParameterError("Parameter duration_time type error")
        if value < 0 or (value > 30000 and value != 65535):
            logger.error("Parameter duration_time out of range (0, 10000) error")
            raise ParameterError("Parameter duration_time out of range (0, 10000) error")
        self._duration_time = value

    @property
    def mode(self):
        return self._mode

    @mode.setter
    def mode(self, value):
        if value is None:
            if self.mode is None:
                logger.error("Parameter mode is None error")
                raise ParameterError("Parameter mode is None error")
            value = self.mode
        if value not in ("always-on", "blink", "breath"):
            logger.error("Parameter mode type error")
            raise ParameterError("Parameter mode type error")
        self._mode = value

    @property
    def rgb(self):
        return self._rgb

    @rgb.setter
    def rgb(self, value):
        if value is None:
            if self._rgb is None:
                logger.error("Parameter rgb is None error")
                raise ParameterError("Parameter rgb is None error")
            value = self.rgb
        if not isinstance(value, tuple):
            logger.error("Parameter rgb type error")
            raise ParameterError("Parameter rgb type error")
        if len(value) is not 3:
            logger.error("Parameter rgb length error")
            raise ParameterError("Parameter rgb length error")
        for i in range(3):
            if value[i] < 0 or value[i] > 255:
                logger.error("Parameter rgb value out of range (0, 255) error")
                raise ParameterError("Parameter rgb value out of range (0, 255) error")
        self._rgb = value

    def set_color(self):
        item = 1001
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.rgb[0], prm2=self.rgb[1], prm3=self.rgb[2])
        subscriber = Subscriber(seq=seq, num=num, item=item, name="set_color", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def set_blink(self):
        item = 1003
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.mode_dict.get(self.mode), prm2=self.duration_time,
                                                prm3=self.light_time, prm4=self.dark_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="set_blink", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def light_off(self):
        item = 1002
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=100)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="light_off", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def light_on(self):
        item = 1002
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=101)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="light_on", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result



