# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import message_center, Subscriber

logger = logging.getLogger("sdk.gomer.motion")


class Motion(Base):
    """Including motion control interface, both open-loop control and closed-loop control, to meet different requirements.
       The closed-loop control, through the movement algorithm, the accurate control mechanism movement to the designated position
    """

    __slots__ = ("_speed", "_hold_time", "_wait_time", "_direction")

    def __init__(self):
        self._speed = 2
        self._hold_time = 5000
        self._wait_time = 3000
        self._direction = 0

    @property
    def direction(self):
        return self._direction

    @direction.setter
    def direction(self, value):
        if value is None:
            if self.direction is None:
                logger.error("Parameter direction is None error")
                raise ParameterError("Parameter direction is None error")
            value = self.direction
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter direction type error")
            raise ParameterError("Parameter direction type error")
        if value < -180 or value > 180:
            logger.error("Parameter speed out of range (-180, 180) error")
            raise ParameterError("Parameter speed out of range (-180, 180) error")
        self._direction = value

    @property
    def speed(self):
        return self._speed

    @speed.setter
    def speed(self, value):
        if value is None:
            value = self.speed
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter speed type error")
            raise ParameterError("Parameter speed type error")
        if value < -3 or value > 3:
            logger.error("Parameter speed out of range (-3, 3) error")
            raise ParameterError("Parameter speed out of range (-3, 3) error")
        self._speed = value

    @property
    def hold_time(self):
        return self._wait_time
    
    @hold_time.setter
    def hold_time(self, value):
        if value is None:
            if self.hold_time is None:
                logger.error("Parameter hold_time is None error")
                raise ParameterError("Parameter hold_time is None error")
            value = self.hold_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter hole_time type error")
            raise ParameterError("Parameter hole_time type error")
        if value < 0 or value > 10000:
            logger.error("Parameter hole_time out of range (0, 10000) error")
            raise ParameterError("Parameter hole_time out of range (0, 10000) error")
        self._hold_time = value
    
    @property
    def wait_time(self):
        return self._wait_time
    
    @wait_time.setter
    def wait_time(self, value):
        if value is None:
            if self.wait_time is None:
                logger.error("Parameter wait_time is None error")
                raise ParameterError("Parameter wait_time is None error")
            value = self.wait_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter wait_time type error")
            raise ParameterError("Parameter wait_time type error")
        if value < 0 or value > 10000:
            logger.error("Parameter wait_time out of range (0, 10000) error")
            raise ParameterError("Parameter wait_time out of range (0, 10000) error")
        self._wait_time = value

    def arm_rotate(self):
        item = 1121
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm3=self.hold_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="arm_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def arm_accurate_rotate(self, angle):
        item = 1122
        try:
            angle = int(angle)
        except Exception:
            logger.error("Parameter angle type error")
            raise ParameterError("Parameter angle type error ")
        if angle < 0 or angle > 170:
            logger.error("Parameter angle out of range (0, 170) error")
            raise ParameterError("Parameter angle out of range (0, 170) error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=angle, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name=" arm_accurate_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def forearm_rotate(self):
        item = 1111
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm3=self.hold_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="forearm_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def forearm_accurate_rotate(self, angle):
        item = 1112
        try:
            angle = int(angle)
        except Exception:
            logger.error("Parameter angle type error")
            raise ParameterError("Parameter angle type error ")
        if angle < 0 or angle > 210:
            logger.error("Parameter angle out of range (0, 210) error")
            raise ParameterError("Parameter angle out of range (0, 210) error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=angle, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="forearm_accurate_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def paw_rotate(self):
        item = 1141
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm3=self.hold_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="paw_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def paw_accurate_rotate(self, angle):
        item = 1142
        try:
            angle = int(angle)
        except Exception:
            logger.error("Parameter angle type error")
            raise ParameterError("Parameter angle type error ")
        if angle < 0 or angle > 100:
            logger.error("Parameter angle out of range (0, 100) error")
            raise ParameterError("Parameter angle out of range (0, 100) error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=angle, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="paw_accurate_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def wheels_move(self):
        item = 1101
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=self.direction, prm3=self.hold_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="wheels_move", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def wheels_accurate_go_straight(self, distance):
        item = 1102
        try:
            distance = int(distance)
        except Exception:
            logger.error("Parameter distance type error")
            raise ParameterError("Parameter distance type error ")
        if distance < -1000 or distance > 1000:
            logger.error("Parameter distance out of range (-1000, 1000) error")
            raise ParameterError("Parameter distance out of range (-1000, 1000) error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=distance, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="wheels_accurate_go_straight", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def wheels_accurate_turn(self, angle):
        item = 1103
        try:
            angle = int(angle)
        except Exception:
            logger.error("Parameter angle type error")
            raise ParameterError("Parameter angle type error ")
        if angle < -360 or angle > 360:
            logger.error("Parameter angle out of range (-360, 360) error")
            raise ParameterError("Parameter angle out of range (-360, 360) error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=angle, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="wheels_accurate_turn", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def head_rotate(self):
        item = 1131
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm3=self.hold_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="head_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def head_accurate_rotate(self, angle):
        item = 1132
        try:
            angle = int(angle)
        except Exception:
            logger.error("Parameter angle type error")
            raise ParameterError("Parameter angle type error ")
        if angle < -15 or angle > 65:
            logger.error("Parameter angle not range between -15 and 65 error")
            raise ParameterError("Parameter angle not range between -15 and 65 error")
        if self.speed < 0:
            logger.error("Parameter speed less than 0  error")
            raise ParameterError("Parameter speed less than 0  error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.speed, prm2=angle, prm3=self.wait_time)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="head_accurate_rotate", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result




