# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import message_center, Subscriber

logger = logging.getLogger("sdk.gomer.pattern")


class Pattern(Base):
    """Pattern recognition interface, open a lot of pattern algorithm data to users"""

    __slots__ = ("_image_type", "_min_size", "_max_size", "_min_score", "_display", "_pattern_index")

    display_dict = {
        True: 101,
        False: 100
    }

    image_type_dict = {
        "camera": 100,
        "image_array": 101,
        "file": 102
    }
    
    def __init__(self):
        self._image_type = "camera"
        self._min_size = 30
        self._max_size = 300
        self._min_score = 10
        self._display = True
        self._pattern_index = None

    @property
    def pattern_index(self):
        return self._pattern_index

    @pattern_index.setter
    def pattern_index(self, value):
        if value is None:
            if self.pattern_index is None:
                logger.error("Parameter pattern_index is None error")
                raise ParameterError("Parameter pattern_index is None error")
            value = self.pattern_index
        self._pattern_index = value

    @property
    def display(self):
        return self._display

    @display.setter
    def display(self, value):
        self._display = value

    @property
    def min_score(self):
        return self._min_score

    @min_score.setter
    def min_score(self, value):
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter min_score type error")
            raise ParameterError("Parameter min_score type error")
        if value < 0 or value > 100:
            logger.error("Parameter min_score not range between 0 and 100 error")
            raise ParameterError("Parameter min_score not range between 0 and 100 error")
        self._min_score = value

    @property
    def max_size(self):
        return self._max_size

    @max_size.setter
    def max_size(self, value):
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter max_size type error")
            raise ParameterError("Parameter max_size type error")
        if value < 30 or value > 300:
            logger.error("Parameter max_size not range between 30 and 300 error")
            raise ParameterError("Parameter max_size not range between 30 and 300 error")
        self._max_size = value

    @property
    def min_size(self):
        return self._min_size

    @min_size.setter
    def min_size(self, value):
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter min_size type error")
            raise ParameterError("Parameter min_size type error")
        if value < 30 or value > 300:
            logger.error("Parameter min_size not range between 30 and 300 error")
            raise ParameterError("Parameter min_size not range between 30 and 300 error")
        self._min_size = value

    @property
    def image_type(self):
        return self._image_type

    @image_type.setter
    def image_type(self, value):
        if value not in ("camera", "image_array", "file"):
            logger.error("Parameter image_type error")
            raise ParameterError("Parameter image_type error")
        self._image_type = value

    def detect_pattern(self):
        item = 3201
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.image_type_dict.get(self.image_type),
                                                prm2=self.min_score, prm3=self.max_size,
                                                prm4=self.min_score, prm5=self.display_dict.get(self.display))
        subscriber = Subscriber(seq=seq, num=num, item=item, name="detect_pattern", parameter="prmstr", msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def add_pattern(self, pattern_path, width, height):
        item = 3202
        if not pattern_path.endswith('.jpg'):
            logger.error("pattern_path format not .jpg error")
            raise ParameterError("pattern_path format not .jpg error")
        try:
            width = int(width)
            height = int(height)
        except Exception:
            logger.error("Parameter width or height type error")
            raise ParameterError("Parameter width or height type error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_transmit_msg(seq, 202, pattern_path, self._get_file_size(pattern_path))
        subscriber = Subscriber(file_path=pattern_path, seq=seq, num=num, item=item, name="add_pattern",
                                msg_center=message_center)
        subscriber.subscribe()
        subscriber.send_msg(msg)
        if subscriber.get_result():
            subscriber.can_upload = 0
            subscriber.file_upload()
            if subscriber.get_result():
                subscriber.upload = 0
                seq = Counter().get_seq()
                msg = self._edit_msg(seq=seq, num=num, item=item, prmstr1=self._path_join(width, height, self._get_file_name(pattern_path)))
                subscriber.seq = seq
                subscriber.send_msg(msg)
                return subscriber.get_result()
            else:
                logger.warning("File upload failed")
                return -1
        else:
            logger.warning("File cannot be uploaded")
            return -1

    def get_3d_position(self):
        item = 3206
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm5=self.display_dict.get(self.display),
                                                prmstr1=self.pattern_index)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="get_3d_position", parameter="prmstr",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def move_to_pattern(self, pattern_name, distance):
        item = 3211
        try:
            distance = int(distance)
        except Exception:
            logger.error("Parameter distance type error")
            raise ParameterError("Parameter distance type error")
        pattern_name = ''.join(('std_c_', pattern_name, '.jpg'))
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=distance, prmstr1=pattern_name)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="move_to_pattern", parameter="prm0",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result







