# -*- coding: utf-8 -*-

from . import config
from .event import Event
from .motion import Motion
from .emotion import Emotion
from .face import Face
from .pattern import Pattern
from .screen import Screen
from .misc import Light
from .audio import Audio
from .conn import DeviceConnector, DeviceCommunication


class Robot(object):
    """Robot class"""
    def __init__(self):
        self._motion = Motion()
        self._emotion = Emotion()
        self._event = Event()
        self._face = Face()
        self._pattern = Pattern()
        self._screen = Screen()
        self._light = Light()
        self._audio = Audio()
        self.event = Event()

    @staticmethod
    def connect(name):
        """Establish communication with  device"""
        connector = DeviceConnector()
        config.SOCKET_ID, sid = connector.connect_device(name)
        communication = DeviceCommunication(config.SOCKET_ID, sid)
        communication.start()

    def make_tts(self, text, file_name=None):
        """TTS and save as file.

        Args:
            text (string): Text messages, up to 128 characters in English and 30 characters in Chinese.
            file_name (string):Save the file name, the file will be saved in gomer brain, can be used as voice, or play.

        Returns:
            returns the filename on success and -1 on failure
        """
        self._audio.file_name = file_name
        return self._audio.make_tts(text)

    def make_music(self, music, file_name=None):
        """Music is synthesized and saved as a file.

        Args:
            music(string): Music spectral information, up to 128 digits.
            file_name(string): Save the file name, the file will be saved in gomer brain, can be used as voice, or play.

        Returns:
            returns the filename on success and -1 on failure
        """
        self._audio.file_name = file_name
        return self._audio.make_music(music)

    def convert_audio(self, src_file, dst_file, convert_type=None):
        """Audio voice, the source audio according to a certain type of voice as the target audio.

        Args:
            src_file(string): Source audio file
            convert_type(string): Variation type.
            dst_file(string): Save the file name, the file will be saved in the gomer brain, can be used for other purposes.

        Returns:
            returns the dst_file on success and -1 on failure
        """
        self._audio.convert_type = convert_type
        return self._audio.convert_audio(src_file, dst_file)

    def get_volume(self):
        """Gets the current volume.

        Returns:
            returns the volume on success and -1 on failure
        """
        return self._audio.get_volume()

    def set_volume(self, vol):
        """Set the volume

        Args:
            vol(int): Set the volume value, ranging from 0 to 5

        Returns:
            returns the volume on success and -1 on failure
        """
        self._audio.vol = vol
        return self._audio.set_volume()

    def get_player_status(self):
        """Gets the current playback status.

        Returns:
            returns the filename if busy, None if idle, and -1 if failed
        """
        return self._audio.get_player_status()

    def play_audio(self, file_name=None):
        """Play an audio file

        Args:
            file_name(string): Play filename

        Returns:
            returns the file_name on success and -1 on failure
        """
        self._audio.file_name = file_name
        return self._audio.play_audio()

    def stop_player(self):
        """Stop the audio player.

        Returns:
            returns None on success and -1 on failure
        """
        return self._audio.stop_player()

    def emotion_show(self, name, expression_switch=None, voice_switch=None, action_switch=None, paw_safe=None):
        """Emotional display, and can freely control the switch of each element.

        Args:
            name(string): The names of emotions, such as "wonderful" or "happy", will randomly select a happy mood.
            expression_switch(bool): The expression switch.
            voice_switch(bool): The sound switch.
            action_switch(bool): Action switch.
            paw_safe(bool): When enabled, the emotional claw action is disabled.

        Returns:
            returns emotion_name on success and -1 on failure
        """
        self._emotion.name = name
        self._emotion.expression_switch = expression_switch
        self._emotion.voice_switch = voice_switch
        self._emotion.action_switch = action_switch
        self._emotion.paw_safe = paw_safe
        return self._emotion.emotion_show()

    def emotion_status(self):
        """Get the current emotional display status

        Returns:
             returns the filename if busy, None if idle, and -1 if failed
        """
        return self._emotion.emotion_status()

    def stop_emotion(self):
        """Stop the current emotional display

        Returns:
             returns None on success and -1 on failure
        """
        return self._emotion.stop_emotion()

    def detect_face(self, image_type=None, min_face_size=None, max_face_size=None, min_face_score=None, display=None):
        """For face recognition, you can select data sources, such as local pictures and cameras, and set face standards.
           Will return information such as face coordinates recognized.

        Args:
            image_type(string): Select data source, Contains" camera"," image_array"," file".
            min_face_size(int): Minimum face size, range 10~300, default 10
            max_face_size(int): Maximum face size, range 10~300, default 300
            min_face_score(int): Minimum face score, range 0~100, default 10, this score is the face of the integrity and
                            clarity of the score.
            display(bool): Display the detection face on the screen.

        Returns:
            returns the face_info_data such as [[x, y, w, h, score],[x, y, w, h, score] on success and -1 on failure
        """
        self._face.image_type = image_type
        self._face.min_face_size = min_face_size
        self._face.max_face_size = max_face_size
        self._face.min_face_score = min_face_score
        self._face.display = display
        return self._face.detect_face()

    def get_face_feature(self, face_index, display=None):
        """To get the coordinates of facial feature points such as eyes, nose, eyebrows, and mouth, the interface
           depends on the result of "face.detect_face".

        Args:
            face_index(string): Select a face index name, such as "face0".
            display(bool): Display the detection face on the screen.

        Returns:
            returns [[lx,ly,rx,ry,nx,ny,lmx,lmy,rmx,rmy],[lx,ly,rx,ry,nx,ny,lmx,lmy,rmx,rmy]...] on success and -1 on failure
        """
        self._face.face_index = face_index
        self._face.display = display
        return self._face.get_face_feature()

    def face_register(self, face_name, face_index=None):
        """Name the Face and enter it into the database. This interface depends on the result of "face.detect_face".

        Args:
            face_index(string): Select a face index name, such as "face0".
            face_name(string): Face names.

        Returns:
            returns face_name on success and -1 on failure
        """
        self._face.face_index = face_index
        return self._face.face_register(face_name)

    def identity_recognize(self, face_index=None, display=None):
        """The face is compared with the robot database to identify the identity.The interface depends on the result of
                    "face.detect_face".

        Args:
            face_index(string): Select a face index name, such as "face0".
            display(bool): Display the detection face on the screen.

        Returns:
            returns identity_name on success and -1 on failure
        """
        self._face.face_index = face_index
        self._face.display = display
        return self._face.identity_recognize()

    def expression_recognize(self, face_index=None, display=None):
        """Facial recognition, the interface depends on the result of "face.detect_face".

        Args:
            face_index(string): Select a face index name, such as "face0".
            display(bool): Display the detection face on the screen.

        Returns:
            returns expression_name on success and -1 on failure
        """
        self._face.face_index = face_index
        self._face.display = display
        return self._face.expression_recognize()

    def get_face_identity(self, identity_name, face_index=None, display=None):
        """Gets the Face's identity data, and the interface depends on the result of "face.detect_face"

        Args:
            face_index(string): Select face index name such as "face0"
            identity_name(string): Name the data and save it as a temporary file
            display(bool): Display the detection face on the screen

        Returns:
            returns identity_name on success and -1 on failure
        """
        self._face.face_index = face_index
        self._face.display = display
        return self._face.get_face_identity(identity_name)

    def get_face_similarity(self, identity_name1, identity_name2):
        """Compare the two faces' identity data. The interface depends on the result of "face.detect_face".

        Args:
            identity_name1(string): The identity data file name, which depends on the result of "face.get_face_identity"
            identity_name2(string): The identity data file name, which depends on the result of "face.get_face_identity"

        Returns:
            returns similarity_data on success and -1 on failure
        """
        return self._face.get_face_similarity(identity_name1, identity_name2)

    def face_rename(self, old_name, new_name):
        """Renaming a face cannot be the same as the name in the database

        Args:
            old_name(string): Existing face names
            new_name(string): New face names

        Returns:
            returns new_name on success and -1 on failure
        """
        return self._face.face_rename(old_name, new_name)

    def get_face_list(self):
        """Get all the faces in the robot database

        Returns:
            returns face_list such as [name1, name2] on success and -1 on failure
        """
        return self._face.get_face_list()

    def set_color(self, rgb=None):
        """Through this interface, you can configure the color of the light, by adjusting the value of the primary colors,
           to achieve any color of the light.

        Args:
            rgb(tuple): color rgb

        Returns:
            returns rgb on success and -1 on failure
        """
        self._light.rgb = rgb
        return self._light.set_color()

    def set_blink(self, mode=None, duration_time=None, light_time=None, dark_time=None):
        """Through this interface, you can configure the flicker mode and duration time.

        Args:
            mode(string): Mode selection, contains ["always-on", " blink", "breathe"]
            duration_time(int): The total duration, ranging from 0 to 30000 ms, is infinite when set to 65535
            light_time(int): The flicker mode is effective. The time of lighting within a flicker cycle ranges from 0 to 10000 ms
            dark_time(int): The flicker mode is effective, and the time for lights to go out within one flicker cycle ranges
                        from 0 to 10000 ms

        Returns:
            returns mode on success and -1 on failure
        """
        self._light.mode = mode
        self._light.duration_time = duration_time
        self._light.light_time = light_time
        self._light.dark_time = dark_time
        return self._light.set_blink()

    def light_off(self) -> object:
        """light off

        Returns:
            returns None on success and -1 on failure
        """
        return self._light.light_off()

    def light_on(self):
        """light on

        Returns:
            returns mode on success and -1 on failure
        """
        return self._light.light_on()

    def arm_rotate(self, speed=None, hold_time=None):
        """Big arm movement, open loop mode, stop sign with time.

        Args:
            speed(int): Motion speed, range -3~3, 0 for stop, positive for backward contraction, negative for forward extension.
            hold_time(int): Motion time, range 0~10000 ms, when time is used up, motion stops.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.hold_time = hold_time
        return self._motion.arm_rotate()

    def arm_accurate_rotate(self, angle, speed=None, wait_time=None):
        """Big arm movement, closed loop control, precise rotation X Angle.

        Args:
            angle(int): Target absolute Angle, range 0~160 degrees, 0 degrees for complete contraction position,
                    160 degrees for complete extension position
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the position,
                        immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.arm_accurate_rotate(angle)

    def forearm_rotate(self, speed=None, hold_time=None):
        """Forearm movement, open loop, stop sign with time.

        Args:
            speed(int): Motion speed, range -3~3, 0 is stop, positive number is outward extend, negative number is inward contract.
            hold_time(int): Motion time, range 0~10000 ms, when time is used up, motion stops.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.hold_time = hold_time
        return self._motion.forearm_rotate()

    def forearm_accurate_rotate(self, angle, speed=None, wait_time=None):
        """Forearm movement, closed loop control, precise rotation X Angle.

        Args:
            angle(int): The absolute Angle of the target ranges from 0 to 210 degrees, 0 degrees for the complete contraction
                    position and 210 degrees for the complete extension position
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the
                        position, immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.forearm_accurate_rotate(angle)

    def paw_rotate(self, speed=None, hold_time=None):
        """Gripper movement, open loop mode, stop sign with time.

        Args:
            speed(int): The range of motion speed is -3~3, 0 is stop, positive number is inward closing, negative number is
                    outward opening.
            hold_time(int): Motion time, range 0~10000 ms, when time is used up, motion stops.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.hold_time = hold_time
        return self._motion.paw_rotate()

    def paw_accurate_rotate(self, angle, speed=None, wait_time=None):
        """Gripper movement, closed loop control, precise rotation X Angle.

        Args:
            angle(int): Target absolute Angle, range 0~100 degrees, 0 degrees is open limit position, 100 degrees is closed
                    limit position.
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the
                        position, immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.paw_accurate_rotate(angle)

    def wheels_move(self, direction=None, speed=None, hold_time=None):
        """Wheel movement, open loop mode, stop sign with time.

        Args:
            direction(int): Direction of motion, range -180~180 degrees, 0 degrees straight ahead.
            speed(int): Motion speed, range 0~ 3,0 is stop.
            hold_time(int): Motion time, range 0~10000 ms, when time is used up, motion stops.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.direction = direction
        self._motion.speed = speed
        self._motion.hold_time = hold_time
        return self._motion.wheels_move()

    def wheels_accurate_go_straight(self, distance, speed=None, wait_time=None):
        """Wheel straight, closed - loop control, accurate to a certain position.

        Args:
            distance(int): Motion distance, range -1000~1000 millimeter, positive number is advance, negative number is
                        retreat.
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the
                        position, immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.wheels_accurate_go_straight(distance)

    def wheels_accurate_turn (self, angle, speed=None, wait_time=None):
        """Wheel turn, closed loop control, precise rotation X Angle.

        Args:
            angle(int): Turn Angle, range -360~360 mm, positive for right turn, negative for left turn
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the
                        position, immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.wheels_accurate_turn(angle)

    def head_rotate(self, speed, hold_time):
        """Head movement, open-loop mode, stop sign with time.

        Args:
            speed(int): The range of motion speed is -3~3, 0 is stop, positive number is up, negative number is down.
            hold_time(int): Motion time, range 0~10000 ms, when time is used up, motion stops.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.hold_time = hold_time
        return self._motion.head_rotate()

    def head_accurate_rotate(self, angle, speed=None, wait_time=None):
        """Head movement, closed loop control, precise rotation X Angle.

        Args:
            angle(int): Absolute Angle of the target, range -15~65 degrees, -15 degrees for the head limit position,
                    0 degrees for the front, 65 degrees for the head limit position.
            speed(int): Motion speed, range 0~ 3,0 is stop.
            wait_time(int): The wait timeout period ranges from 0 to 10,000 milliseconds. When the motion reaches the
                        position, immediately return; Automatically returns when time runs out.

        Returns:
            returns None on success and -1 on failure
        """
        self._motion.speed = speed
        self._motion.wait_time = wait_time
        return self._motion.head_accurate_rotate(angle)

    def detect_pattern(self, image_type=None, min_size=None, max_size=None, min_score=None, display=None):
        """For pattern recognition, you can select data sources, such as local pictures and cameras, and set pattern
            standards. The recognized pattern information will be returned.

        Args:
            image_type(string): Select data source, Contains "camera","image_array","file"
            min_size(int): Minimum pattern size, range 30~300, default 30
            max_size(int): Maximum pattern size, range 30~300, default 300
            min_score(int): Minimum pattern score, range 0~100, default 10, this score is for pattern integrity and clarity
            display(bool): Display the detection pattern on the screen

        Returns:
            returns the pattern_data such as [[lux,luy,ldx,ldy,rdx,rdy,rux,ruy,score],...] on success and -1 on failure
        """
        self._pattern.image_type = image_type
        self._pattern.min_size = min_size
        self._pattern.max_size = max_size
        self._pattern.min_score = min_score
        self._pattern.display = display
        return self._pattern.detect_pattern()

    def add_pattern(self, pattern_path, width, height):
        """Add a pattern from PC

        Args:
            pattern_path(string): The absolute path of pattern file , pattern name should be less than 16 characters
            width(int): Pattern width, in centimeters
            height(int): Pattern hight, in centimeters

        Returns:
            returns the pattern_name on success and -1 on failure
        """
        return self._pattern.add_pattern(pattern_path, width, height)

    def get_3d_position(self, pattern_index=None, display=None):
        """Gets the 3D orientation of the Pattern in the current screen. This interface depends on the result of
            "Pattern. Detect_pattern ".

        Args:
            pattern_index(string): The index name of the pattern
            display(bool): Display the detection pattern on the screen

        Returns:
            returns the position_data such as [[x,y,z,theta_x,theta_y,theta_z],...] on success and -1 on failure
        """
        self._pattern.pattern_index = pattern_index
        self._pattern.display = display
        return self.get_3d_position()

    def move_to_pattern(self, pattern_name, distance):
        """Move to the front of the pattern and keep X distance.

                Args:
                    pattern_name(string): Pattern name contains " 'G' ,'L' ,'I' ,'A','B','C' ......"
                    distance(int): The distance from the pattern in centimeters

                Returns:
                    returns the position_data such as [prm1, prm2, prm3, prm4, prm5] on success and -1 on failure
                                        prm1: If the value is 101, it means in place; otherwise, it is not in place
                                        prm2: Represents the position where the head still needs to be moved, and a value of 255
                                                indicates that the head is in place
                                        prm3: Represents the position where the wheel needs to turn for the first time.
                                                If the value is 0, the steering position is in place.
                                        prm4: Represents the position where the wheel still needs to go straight.
                                                If the value is 0, it means that the wheel is in place
                                        prm5: Represents the position where the wheel needs to turn for the second time.
                                                If the value is 0, the steering position is in place.
                """
        return self._pattern.move_to_pattern(pattern_name, distance)

    def show_picture(self, img_file, hold_time=None):
        """Display the user's picture on the screen.

        Args:
            img_file(string): The absolute path of img file,The image must be in JPG format with 320x240 resolution.
            hold_time(int): Picture display time, ranging from 1000 to 10000 milliseconds.

        Returns:
            returns the picture_name on success and -1 on failure
        """
        self._screen.hold_time = hold_time
        return self._screen.show_picture(img_file)

    def show_status(self):
        """Get display status

        Returns:
            returns the filename if busy, None if idle, and -1 if failed
        """
        return self._screen.show_status()

    def mirror_camera(self):
        """The camera's image is projected onto the screen and will remain open until the user closes.

        Returns:
            returns None on success and -1 on failure
        """
        return self._screen.mirror_camera()

    def stop_mirror_camera(self):
        """Stop camera projection

        Returns:
            returns None on success and -1 on failure
        """
        return self._screen.stop_mirror_camera()

    def stop_showing(self):
        """Stop the screen display.

        Returns:
            returns None on success and -1 on failure
        """
        return self._screen.stop_showing()






