# -*- coding: utf-8 -*-

import logging
from .base import Base
from .utils import Counter
from .exceptions import ParameterError
from .handler import Subscriber, message_center
from .gproto import file_upload
from .exceptions import FilePathError

logger = logging.getLogger("sdk.gomer.screen")


class Screen(Base):
    """Screen display class, provides a variety of display methods, including user - defined pattern display."""

    __slots__ = ("_hold_time", )
    
    def __init__(self):
        self._hold_time = 5000
        
    @property
    def hold_time(self):
        return self._hold_time
    
    @hold_time.setter
    def hold_time(self, value):
        if not value:
            value = self.hold_time
        try:
            value = int(value)
        except Exception:
            logger.error("Parameter hold_time type error")
            raise ParameterError("Parameter hold_time type error")
        if value < 1000 or value > 10000:
            logger.error("Parameter hold_time not range between 1000 and 10000 error")
            raise ParameterError("Parameter hold_time not range between 1000 and 10000 error")
        self._hold_time = value

    def show_picture(self, img_file):
        item = 1301
        if not img_file.endswith('.jpg'):
            logger.error("Image format not .jpg error")
            raise ParameterError("Image format not .jpg error")

        if not self._isfile(img_file):
            logger.error("%s is the wrong file path" % img_file)
            raise FilePathError("%s is the wrong file path" % img_file)
        file_name = self._get_file_name(img_file)
        file_size = self._get_file_size(img_file)
        if file_size >= 5 * 1024 * 1024:
            logger.error("File greater than 5M error")
            raise FilePathError("File greater than 5M error")
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_transmit_msg(seq, 202, img_file, file_size)
        subscriber = Subscriber(file_path=img_file, seq=seq, num=num, item=item, name="show_picture", msg_center=message_center)
        subscriber.subscribe()
        subscriber.send_msg(msg)
        if subscriber.get_result():
            subscriber.can_upload = 0
            file_upload(img_file)
            if subscriber.get_result():
                subscriber.upload = 0
                seq = Counter().get_seq()
                msg = self._edit_msg(seq=seq, num=num, item=item, prm1=self.hold_time,
                                     prmstr1=file_name)
                subscriber.seq = seq
                subscriber.send_msg(msg)
                return subscriber.get_result()
            else:
                logger.warning("File upload failed")
                return -1
        else:
            logger.warning("File cannot be uploaded")
            return -1

    def show_status(self):
        item = 1302
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="show_status",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def mirror_camera(self):
        item = 1303
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=101)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="mirror_camera",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def stop_mirror_camera(self):
        item = 1303
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item, prm1=100)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="stop_mirror_camera",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result

    def stop_showing(self):
        item = 1304
        seq = Counter().get_seq()
        num = Counter().get_num()
        msg = self._edit_msg(seq=seq, num=num, item=item)
        subscriber = Subscriber(seq=seq, num=num, item=item, name="stop_showing",
                                msg_center=message_center)
        result = self._message_transmit(msg, subscriber)
        return result


